part of persik;

@NgController()
class AppController {
  List<Item> items;
  Item newItem;
  DateTime date;
  bool addingItems;
  Login login;
  String error = "";
  User currentUser;

  DateFormat _formatter;
  Backend _backend;
  Map<String, String> _params;
  
  AppController(Scope scope) {
    newItem = new Item.fromJson(null);
    login = new Login();
    items = new List<Item>();

    _backend = new Backend();
    _formatter = new DateFormat('EEE d MMMM yyyy');
    _params = Utils.parseUrlParameters();

    String possibleDate = _params["date"];
    if (possibleDate != null && possibleDate.isNotEmpty) {
      int unix = int.parse(possibleDate);
      date = new DateTime.fromMillisecondsSinceEpoch(unix, isUtc: false);
    } else {
      date = new DateTime.now();
    }

    if (!_finishOauthLogin()) {
      _fetchCurrentUser();
      _fetchItems();
    }
  }

  bool _finishOauthLogin() {
    if (!_params.containsKey("state") || !_params.containsKey("code")) {
      return false;
    }
    String url = Utils.urlWithoutSearch();
    _backend.postOauthLogin(_params["state"], _params["code"])
      .then((User user) {
        window.history.pushState({"oauth_login": _params["state"]}, document.title, url);
        _setUser(user);
      })
      .catchError((e) {
        window.location.replace(url);
      });
    return true;
  }

  void add() {
    if (newItem.isEmpty) {
      return;
    }
    items.add(newItem);
    if (null != currentUser) {
      _backend.postItem(newItem).catchError(_showError);
    }
    newItem = new Item.fromJson(null);
  }

  String classFor(Item item) => item.finished_at != null ? 'done' : '';

  int remaining() => items.where((item) => item.finished_at == null).length;

  int finished() => items.where((item) => item.finished_at != null).length;

  double successRate() => 100 * finished() / items.length;

  String formattedDate() => _formatter.format(date);

  bool startAddingItems() => addingItems = true;

  bool doneAddingItems() => addingItems = false;

  void signin() {
    error = "";
    _backend.postSession(login)
      .then(_setUser)
      .catchError((e) {
        login.password = "";
        _showError(e);
      });
  }

  void _setUser(User user) {
    login.password = "";
    currentUser = user;
    _fetchItems();
  }

  void startOauthLogin(String service) {
    error = "";
    _backend.getOauthLoginUrl(service)
      .then(window.location.replace)
      .catchError(_showError);
  }

  void register() {
    error = "";
    _backend.postUser(login)
      .then((User user) {
        login.password = "";
        currentUser = user;
        _backend.postBulkItems(items).catchError(_showError);
      })
      .catchError((e) {
        login.password = "";
        _showError(e);
      });
  }

  void signout() {
    error = "";
    currentUser = null;
    items.clear();
    items.addAll(Item.DefaultItems);
    _backend.deleteSession().catchError(_showError);
  }

  void _fetchCurrentUser() {
    _backend.getCurrentUser()
      .then((User user) {
        currentUser = user;
      })
      .catchError((ErrorResponse err){
        if (403 != err.status) {
          _showError(err);
        }
      });
  }
  
  void _showError(ErrorResponse err) {
    error = err.text;
  }

  void _fetchItems() {
    _backend.getItems(date)
      .then((List<Item> list){
        items = list;
      })
      .catchError((ErrorResponse err){
        if (403 != err.status) {
          return _showError(err);
        }
        items.clear();
        items.addAll(Item.DefaultItems);
      });
  }

  bool get hasError => error.isNotEmpty;

  String clearError() => error = "";

  void checkItem(Item item) {
    if (currentUser != null) {
      _backend.putItem(item).catchError(_showError);
    }
  }
}
