part of persik;

class ErrorResponse {
  String text;
  int status;
}

class Backend {
  static const String url = "http://localhost:8080/api";
  //static const String url = "/api";

  ErrorResponse _parseError(ProgressEvent pe) {
    HttpRequest req = pe.target;
    ErrorResponse result = new ErrorResponse();
    result.status = req.status;
    result.text = req.responseText;
    if (result.text.isEmpty) {
      result.text = req.statusText;
    }
    if (result.text.isNotEmpty) {
      result.text = result.text.substring(0, 1).toUpperCase() + result.text.substring(1);
    }
    return result;
  }

  Future<User> postUser(Login login) {
    var completer = new Completer();
    HttpRequest
      .request(url+"/users", method: "POST", sendData: JSON.encode(login), withCredentials: true)
      .then((HttpRequest req) {
        completer.complete(new User.fromJSON(JSON.decode(req.response)));
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future<List<Item>> getItems(DateTime date) {
    DateTime startTime = new DateTime(date.year, date.month, date.day, 0, 0, 0, 0);
    DateTime endTime = startTime.add(new Duration(hours: 24));

    String itemsUrl = url+"/items";
    itemsUrl += "?start_millis=" + startTime.millisecondsSinceEpoch.toString();
    itemsUrl += "&end_millis=" + endTime.millisecondsSinceEpoch.toString();

    var completer = new Completer();
    HttpRequest.getString(itemsUrl, withCredentials: true)
      .then((String data) {
        completer.complete(Item.listFromJSONString(data));
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future<Item> postItem(Item item) {
    var completer = new Completer();
    HttpRequest
      .request(url+"/items", method: "POST", sendData: JSON.encode(item), withCredentials: true)
      .then((HttpRequest req) {
        completer.complete(new Item.fromJson(JSON.decode(req.response)));
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future<Item> putItem(Item item) {
    var completer = new Completer();
    HttpRequest
      .request(url+"/items/"+item.id,
          method: "PUT",
          sendData: JSON.encode(item),
          withCredentials: true)
      .then((HttpRequest req) {
        completer.complete(new Item.fromJson(JSON.decode(req.response)));
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future<List<Item>> postBulkItems(List<Item> list) {
    var completer = new Completer();
    HttpRequest
      .request(url+"/bulk/items", method: "POST", sendData: JSON.encode(list), withCredentials: true)
      .then((HttpRequest req) {
        completer.complete(Item.listFromJSONString(req.response));
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future<User> postSession(Login login) {
    var completer = new Completer();
    HttpRequest
      .request(url+"/sessions", method: "POST", sendData: JSON.encode(login), withCredentials: true)
      .then((HttpRequest req) {
        completer.complete(new User.fromJSON(JSON.decode(req.response)));
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future deleteSession() {
    var completer = new Completer();
    HttpRequest
      .request(url+"/session", method: "DELETE", withCredentials: true)
      .then((_) {
        completer.complete();
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future<User> getCurrentUser() {
    var completer = new Completer();
    HttpRequest.getString(url+"/current_user", withCredentials: true)
      .then((String data) {
        User user = null;
        if (data != "null") {
          user = new User.fromJSON(JSON.decode(data));
        }
        completer.complete(user);
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future<String> getOauthLoginUrl(String service) {
    var completer = new Completer();
    HttpRequest.getString(url+"/oauth_login_url?service=" + service)
      .then((String data) {
        var response = JSON.decode(data);
        completer.complete(response["url"]);
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }

  Future<User> postOauthLogin(String service, String code) {
    Map<String, String> payload = new Map<String, String>();
    payload["service"] = service;
    payload["code"] = code;

    var completer = new Completer();
    HttpRequest
      .request(url+"/oauth_login", method: "POST", sendData: JSON.encode(payload), withCredentials: true)
      .then((HttpRequest req) {
        completer.complete(new User.fromJSON(JSON.decode(req.response)));
      })
      .catchError((e) {
        completer.completeError(_parseError(e));
      });
    return completer.future;
  }
}
